/* 
Script for Kero Follower V2.0
	by Steel Titanium, adapted by Bartman3010
	
Creates a Kero object to follow Sakura Kinomoto and react to the character in-game.
*/

//Freeslot the required stuff
freeslot(
	"SPR_KRFL",
	"S_KEROIDLE",
	"S_KEROFLY",
	"S_KEROWIN1",
	"S_KEROWIN2",
	"S_KEROWIN3",
	"S_KEROPANIC1",
	"S_KEROPANIC2",
	"S_KEROSAD",
	"MT_KERO"
)

mobjinfo[MT_KERO] = {
	doomednum = -1,
	spawnstate = S_KEROIDLE,
	spawnhealth = 1,
	painchance = 200,
	speed = 3,
	radius = 16*FRACUNIT,
	height = 32*FRACUNIT,
	dispoffset = 0,
	mass = 100,
	damage = 0,
	reactiontime = 32,
	activesound = sfx_None,
	flags = MF_NOGRAVITY|MF_FLOAT|MF_SCENERY|MF_NOCLIPHEIGHT|MF_DONTENCOREMAP,
	raisestate = S_NULL
}

states[S_KEROIDLE] = {
	sprite = SPR_KRFL,
	frame = A,
	tics = -1,
	var1 = 1,
	var2 = 2,
	nextstate = S_KEROIDLE
}

states[S_KEROFLY] = {
	sprite = SPR_KRFL,
	frame = B,
	tics = -1,
	var1 = 1,
	var2 = 2,
	nextstate = S_KEROFLY
}


states[S_KEROWIN1] = {
	sprite = SPR_KRFL,
	frame = C,
	tics = 16,
	nextstate = S_KEROWIN2
}

states[S_KEROWIN2] = {
	sprite = SPR_KRFL,
	frame = D,
	tics = 4,
	nextstate = S_KEROWIN3
}

states[S_KEROWIN3] = {
	sprite = SPR_KRFL,
	frame = E,
	tics = 8,
	nextstate = S_KEROWIN1
}

states[S_KEROPANIC1] = {
	sprite = SPR_KRFL,
	frame = F,
	tics = 6,
	nextstate = S_KEROPANIC2
}

states[S_KEROPANIC2] = {
	sprite = SPR_KRFL,
	frame = G,
	tics = 6,
	nextstate = S_KEROPANIC1
}

states[S_KEROSAD] = {
	sprite = SPR_KRFL,
	frame = H,

}

addHook("MobjThinker", function(mo)
	if mo and mo.skin == "sakura_kero_ccs" and mo.player //Skin check
		local player = mo.player
		if not player.mo.kero
			player.mo.kero = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_KERO)
			player.mo.kero.angle = player.mo.angle
			player.mo.kero.flags2 = player.mo.flags2
			player.mo.kero.destscale = player.mo.destscale
			player.mo.kero.target = player.mo
		end
	elseif mo and mo.skin ~= "sakura_kero_ccs" and mo.kero and mo.kero.valid then //Not using CCSakura but still has a Kero with them.
		P_RemoveMobj(mo.kero)
		mo.kero = nil
	end
end, MT_PLAYER)

addHook("MobjThinker", function(mo)
	-- Thinker for Kero
	if mo and mo.valid then
		if mo.target and mo.target.valid and mo.target.player and mo.target.player.valid then
			local tg = mo.target
			local totz
			local newx
			local newy
			local mangle = FixedAngle(((leveltime * 5) % 360) << FRACBITS)
			
			mo.angle = tg.angle
			/*mo.momx*/ newx = sin(mangle) * (tg.radius>>FRACBITS) * 2
			/*mo.momy*/ newy = cos(mangle) * (tg.radius>>FRACBITS) * 2

			-- Colorize Kero if the player has invincibility, grow etc.
			mo.color = tg.color
			mo.colorized = tg.colorized

			-- Check for gravity flip
			if (mo.eflags & MFE_VERTICALFLIP)
				totz = (tg.z) - (tg.height)
			else
				totz = (tg.z) + (tg.height)
			end
			-- Kero's rotation position when player is idle
			
			mo.destscale = tg.destscale
			mo.angle = tg.angle
			mo.eflags = tg.eflags
			mo.flags2 = tg.flags2

			if tg.player.kartstuff[k_spinouttimer] > 0 or tg.health == 0
				if not (mo.state >= S_KEROPANIC1 and mo.state <= S_KEROPANIC2)
					mo.state = S_KEROPANIC1
				end
				mo.z = totz
				P_MoveOrigin(mo, tg.x, tg.y, totz)
				mo.angle = tg.angle
			end

			if tg.health > 0 and tg.player.kartstuff[k_spinouttimer] == 0 then
				if tg.player.speed > 0 or (tg.momx + tg.momy + tg.momz) > 0 then // If player is moving
					if mo.prevtpos == nil then
						mo.prevtpos = {}
					end

					if not (mo.state == S_KEROFLY)
						mo.state = S_KEROFLY
					end
					local trailby = 2

					if #mo.prevtpos >= trailby then
						local e = mo.prevtpos[trailby]
						local etotz

						if (mo.eflags & MFE_VERTICALFLIP)
							etotz = e.z - (tg.height) - (sin(FixedAngle(leveltime*5*FRACUNIT))*8)
						else
							etotz = e.z + (tg.height) + (sin(FixedAngle(leveltime*5*FRACUNIT))*8)
						end

						mo.prevtpos[trailby] = nil
						P_MoveOrigin(mo, e.x, e.y, etotz)
					end

					table.insert(mo.prevtpos, 1, {x = tg.x, y = tg.y, z = tg.z})
				else -- If player is idle
					P_MoveOrigin(mo, newx + tg.x, newy + tg.y, totz)

					if tg.player.exiting > 0 and K_IsPlayerLosing(tg.player) == false
						if not (mo.state >= S_KEROWIN1 and mo.state <= S_KEROWIN3)
							mo.state = S_KEROWIN1
						end
					elseif tg.player.exiting > 0 and K_IsPlayerLosing(tg.player) == true
						if not (mo.state == S_KEROSAD)
							mo.state = S_KEROSAD
						end
					else
						if not (mo.state == S_KEROIDLE)
							mo.state = S_KEROIDLE
						end
					end


				end
			end
		end
	end

	if mo and (not mo.target or not mo.target.valid) or (not mo.target.player or not mo.target.valid) then
		P_RemoveMobj(mo)
    end

end, MT_KERO)